// sintassi: nome_file

#include <windows.h>
#include <stdio.h>

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva);

int main(int argc, char *argv[]) 
{
	HANDLE hFile;
	BYTE *BaseAddress;
	DWORD FileSize, BR, Bound_Offset;

	IMAGE_DOS_HEADER *ImageDosHeader;
	IMAGE_NT_HEADERS *ImageNtHeaders;

	IMAGE_BOUND_IMPORT_DESCRIPTOR *ImgBoundDescr;
	IMAGE_BOUND_FORWARDER_REF *ImgForwRef;

	DWORD Size = 0, x;

	char *ModName;

	// controlla numero argomenti
	if (argc < 2) 
	{
		printf("\nNeed More Arguments\n");
		return -1;
	}
 
	printf("\nOpening File...\n"); 

	hFile = CreateFile(argv[1], GENERIC_READ, FILE_SHARE_READ, 
		0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

	if (hFile == INVALID_HANDLE_VALUE) 
	{
		printf("Cannot Open the File\n");
		return -1;
	}

	FileSize = GetFileSize(hFile, NULL);

	BaseAddress = (BYTE *) malloc(FileSize);

	if (!ReadFile(hFile, BaseAddress, FileSize, &BR, NULL))
	{
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}
 
	ImageDosHeader = (IMAGE_DOS_HEADER *) BaseAddress;

	// controlliamo il Dos Header
	if (ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		printf("Invalid Dos Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}

	ImageNtHeaders = (IMAGE_NT_HEADERS *) 
		(ImageDosHeader->e_lfanew + (DWORD) ImageDosHeader);

	// controlliamo il PE Header
	if (ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		printf("Invalid PE Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	if (!ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_BOUND_IMPORT].VirtualAddress)
	{
		printf("The IT of this PE isn't bound\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	} 

	// converte in offset fisico l'RVA
	Bound_Offset = RvaToOffset(ImageNtHeaders, 
		ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_BOUND_IMPORT].VirtualAddress);

	//  valido?
	if (Bound_Offset == NULL)
	{
		printf("The IT of this PE isn't bound\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	printf("\nBinding Info:");

	while (TRUE)
	{
		// prendo il Bound Descr
		ImgBoundDescr = (IMAGE_BOUND_IMPORT_DESCRIPTOR *)
			(DWORD) (Bound_Offset + Size + (DWORD) BaseAddress);

		// finiti i descriptors?
		if (!ImgBoundDescr->OffsetModuleName)
			break;

		// ricavo il nome basandomi
		// sull'inizio della directory
		ModName = (char *) (DWORD) 
			(ImgBoundDescr->OffsetModuleName +
			Bound_Offset + (DWORD) BaseAddress);

		// stampo le info
		printf("\n\nModule Name: %s\n"
			"Time/Date Stamp: %08X\n"
			"Forwarder Mods: %d", ModName,
			ImgBoundDescr->TimeDateStamp,
			ImgBoundDescr->NumberOfModuleForwarderRefs);

		Size += sizeof (IMAGE_BOUND_IMPORT_DESCRIPTOR);

		// ci sono forwarder modules?

		if (ImgBoundDescr->NumberOfModuleForwarderRefs)
			printf("\n\nForwarder Modules:");

		for (x = 0; x < ImgBoundDescr->NumberOfModuleForwarderRefs; x++)
		{
			// vabbe stessa cosa di prima solo fatto
			// per i forwarder mod
			ImgForwRef = (IMAGE_BOUND_FORWARDER_REF *) (DWORD)
				(Bound_Offset + Size + (DWORD) BaseAddress);

			ModName = (char *) (DWORD) 
				(ImgForwRef->OffsetModuleName +
				Bound_Offset + (DWORD) BaseAddress);

			printf("\n\nModule Name: %s\n"
				"Time/Date Stamp: %08X", ModName,
				ImgForwRef->TimeDateStamp);

			Size += sizeof (IMAGE_BOUND_FORWARDER_REF);
		}
	}

	free(BaseAddress);
	CloseHandle(hFile);

	return 0;
}

// sappiamo a cosa serve

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva)
{
	DWORD Offset = Rva, Limit;
	IMAGE_SECTION_HEADER *Img;
	WORD i;

	Img = IMAGE_FIRST_SECTION(NT);

	if (Rva < Img->PointerToRawData)
		return Rva;

	for (i = 0; i < NT->FileHeader.NumberOfSections; i++)
	{
		if (Img[i].SizeOfRawData)
			Limit = Img[i].SizeOfRawData;
		else
			Limit = Img[i].Misc.VirtualSize;

		if (Rva >= Img[i].VirtualAddress &&
		    Rva < (Img[i].VirtualAddress + Limit))
		{
			if (Img[i].PointerToRawData != 0)
			{
				Offset -= Img[i].VirtualAddress;
				Offset += Img[i].PointerToRawData;
			}
				
			return Offset;
		}
	}

	return NULL;
}