// sintassi: nome_file

#include <windows.h>
#include <stdio.h>

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva);

int main(int argc, char *argv[]) 
{
	HANDLE hFile;
	BYTE *BaseAddress;
	DWORD FileSize, BR, ET_Offset;

	IMAGE_DOS_HEADER *ImageDosHeader;
	IMAGE_NT_HEADERS *ImageNtHeaders;
	IMAGE_EXPORT_DIRECTORY *ImageExportDir;

	DWORD *Functions, *Names;
	WORD *NameOrds, x, y;
	char *Name, *FName;


	// controlla numero argomenti
	if (argc < 2) 
	{
		printf("\nNeed More Arguments\n");
		return -1;
    }
 
	printf("\nOpening File...\n"); 

	hFile = CreateFile(argv[1], GENERIC_READ | GENERIC_WRITE, 
		FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

	if (hFile == INVALID_HANDLE_VALUE) 
	{
		printf("Cannot Open the File\n");
		return -1;
	}

	FileSize = GetFileSize(hFile, NULL);

	BaseAddress = (BYTE *) malloc(FileSize);

	if (!ReadFile(hFile, BaseAddress, FileSize, &BR, NULL))
	{
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}
 
	ImageDosHeader = (IMAGE_DOS_HEADER *) BaseAddress;

	// controlliamo il Dos Header
	if (ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		printf("Invalid Dos Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}

	ImageNtHeaders = (IMAGE_NT_HEADERS *) 
		(ImageDosHeader->e_lfanew + (DWORD) ImageDosHeader);

	// controlliamo il PE Header
	if (ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		printf("Invalid PE Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	if (!ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress)
	{
		printf("This PE Doesn't contain an ET\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	} 

	// converte in offset fisico l'RVA
	ET_Offset = RvaToOffset(ImageNtHeaders, 
		ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress);

	//  valido?
	if (ET_Offset == NULL)
	{
		printf("This PE Doesn't contain an ET\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	printf("Export Table:\n");

	// ricava la Export Dir
	ImageExportDir = (IMAGE_EXPORT_DIRECTORY *) (ET_Offset +
		(DWORD) BaseAddress);

	Name = (char *) (RvaToOffset(ImageNtHeaders, 
		ImageExportDir->Name) + (DWORD) BaseAddress);

	// stampa le info importanti dell'ET
	printf("\n\nName:                %s\n"
		   "Base:                %08X\n"
		   "Number Of Functions: %08X\n"
		   "Number Of Names:     %08X\n"
		   "Addr Of Functions:   %08X\n"
		   "Addr Of Names:       %08X\n"
		   "Addr Of Name Ords:   %08X\n\n"
		   "Exports:\n", Name, 
		   ImageExportDir->Base,
		   ImageExportDir->NumberOfFunctions,
		   ImageExportDir->NumberOfNames,
		   ImageExportDir->AddressOfFunctions,
		   ImageExportDir->AddressOfNames,
		   ImageExportDir->AddressOfNameOrdinals);


	// prende i vari arrays

	Functions = (DWORD *) (RvaToOffset(ImageNtHeaders, 
		ImageExportDir->AddressOfFunctions) + (DWORD) BaseAddress);

	Names = (DWORD *) (RvaToOffset(ImageNtHeaders, 
		ImageExportDir->AddressOfNames) + (DWORD) BaseAddress);

	NameOrds = (WORD *) (RvaToOffset(ImageNtHeaders, 
		ImageExportDir->AddressOfNameOrdinals) + (DWORD) BaseAddress);

	// enumera le funzioni

	for (x = 0; x < ImageExportDir->NumberOfFunctions; x++)
	{
		// controllo se l'EP  0
		// se s allora passa alla prossima funzione
		if (Functions[x] == 0)
			continue;

		printf("\nOrd:  %04X\nEP:   %08X\n", 
			(x + ImageExportDir->Base), Functions[x]);

		// vedo se la funzione ha anche un nome
		for (y = 0; y < ImageExportDir->NumberOfNames; y++)
		{
			// trovata una funzione con nome?
			if (NameOrds[y] == x)
			{
				FName = (char *) (RvaToOffset(ImageNtHeaders, 
					Names[y]) + (DWORD) BaseAddress);
				printf("Name: %s\n", FName);
				break;
			}
		}
	}

	free(BaseAddress);
	CloseHandle(hFile);

	return 0;
}

// sappiamo a cosa serve

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva)
{
	DWORD Offset = Rva, Limit;
	IMAGE_SECTION_HEADER *Img;
	WORD i;

	Img = IMAGE_FIRST_SECTION(NT);

	if (Rva < Img->PointerToRawData)
		return Rva;

	for (i = 0; i < NT->FileHeader.NumberOfSections; i++)
	{
		if (Img[i].SizeOfRawData)
			Limit = Img[i].SizeOfRawData;
		else
			Limit = Img[i].Misc.VirtualSize;

		if (Rva >= Img[i].VirtualAddress &&
		    Rva < (Img[i].VirtualAddress + Limit))
		{
			if (Img[i].PointerToRawData != 0)
			{
				Offset -= Img[i].VirtualAddress;
				Offset += Img[i].PointerToRawData;
			}
				
			return Offset;
		}
	}

	return NULL;
}