// sintassi: nome_file

#include <windows.h>
#include <stdio.h>

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva);

char *ResNames[] = {
	"CURSORS", "BITMAPS", "ICONS", "MENUS", "DIALOGS", 
	"STRING TABLES", "FONT DIRECORY", "FONTS", "ACCELERATORS",
	"RCDATA", "MESSAGE TABLES", "CURSOR GROUPS"
};

int main(int argc, char *argv[]) 
{
	HANDLE hFile;
	BYTE *BaseAddress;
	DWORD FileSize, BR, Res_Offset;

	IMAGE_DOS_HEADER *ImageDosHeader;
	IMAGE_NT_HEADERS *ImageNtHeaders;

	IMAGE_RESOURCE_DIRECTORY *ImgResDir;
	IMAGE_RESOURCE_DIRECTORY_ENTRY *ImgResDirEntry;

	DWORD ResDirs, x;

	IMAGE_RESOURCE_DIR_STRING_U *uString;
	char DirName[100];


	// controlla numero argomenti
	if (argc < 2) 
	{
		printf("\nNeed More Arguments\n");
		return -1;
    }
 
	printf("\nOpening File...\n"); 

	hFile = CreateFile(argv[1], GENERIC_READ, FILE_SHARE_READ, 
		0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

	if (hFile == INVALID_HANDLE_VALUE) 
	{
		printf("Cannot Open the File\n");
		return -1;
	}

	FileSize = GetFileSize(hFile, NULL);

	BaseAddress = (BYTE *) malloc(FileSize);

	if (!ReadFile(hFile, BaseAddress, FileSize, &BR, NULL))
	{
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}
 
	ImageDosHeader = (IMAGE_DOS_HEADER *) BaseAddress;

	// controlliamo il Dos Header
	if (ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		printf("Invalid Dos Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}

	ImageNtHeaders = (IMAGE_NT_HEADERS *) 
		(ImageDosHeader->e_lfanew + (DWORD) ImageDosHeader);

	// controlliamo il PE Header
	if (ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		printf("Invalid PE Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	if (!ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_RESOURCE].VirtualAddress)
	{
		printf("This PE Doesn't Contain Resources\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	} 

	// converte in offset fisico l'RVA
	Res_Offset = RvaToOffset(ImageNtHeaders, 
		ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_RESOURCE].VirtualAddress);

	//  valido?
	if (Res_Offset == NULL)
	{
		printf("This PE Doesn't Contain Resources\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	// ricavo la Resource Dir
	ImgResDir = (IMAGE_RESOURCE_DIRECTORY *)(DWORD)
		(Res_Offset + (DWORD) BaseAddress);

	// ricavo il numero delle Resource Dirs
	ResDirs = ImgResDir->NumberOfIdEntries +  
		ImgResDir->NumberOfNamedEntries;

	printf("\nNumber of Resource Directories: %d\n", ResDirs);

	// ricavo il puntatore alle Res Dirs
	ImgResDirEntry = (IMAGE_RESOURCE_DIRECTORY_ENTRY *)
		(DWORD) (sizeof (IMAGE_RESOURCE_DIRECTORY) + (DWORD) ImgResDir);

	printf("\nDirectories:\n\n");

	// mostro il 'secondo livello'
	for (x = 0; x < ResDirs; x++)
	{
		printf("\nDirectory %d", (x + 1)); 

		// la dir ha nome?
		if (ImgResDirEntry[x].NameIsString == TRUE)
		{
			uString = (IMAGE_RESOURCE_DIR_STRING_U *)
				(DWORD) (ImgResDirEntry[x].NameOffset +
				(DWORD) ImgResDir);

			ZeroMemory(DirName, sizeof(DirName));

			// converto la stringa unicode
			WideCharToMultiByte(CP_ACP, NULL, 
				(LPCWSTR) &uString->NameString, uString->Length, 
				DirName, sizeof (DirName), NULL, NULL); 

			printf(" Name: %s\n", DirName);
		}
		else
		{
			// stampa l'ID
			printf(" ID: %d", ImgResDirEntry[x].Id);

			// controllo se l'ID risulta tra quelli identificati
			if (ImgResDirEntry[x].Id > 0 &&
				ImgResDirEntry[x].Id < 13)
			{
				printf(" - %s", ResNames[ImgResDirEntry[x].Id - 1]);
			}
			else if (ImgResDirEntry[x].Id == 14)
			{
				printf(" - ICON GROUPS");
			}
			else if (ImgResDirEntry[x].Id == 16)
			{
				printf(" - VERSION INFO");
			}
			else if (ImgResDirEntry[x].Id == 23)
			{
				printf(" - HTML PAGES");
			}
			else if (ImgResDirEntry[x].Id == 24)
			{
				printf(" - CONFIGURATION FILES");
			}

			printf("\n");
		}
	}

	free(BaseAddress);
	CloseHandle(hFile);

	return 0;
}

// sappiamo a cosa serve

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva)
{
	DWORD Offset = Rva, Limit;
	IMAGE_SECTION_HEADER *Img;
	WORD i;

	Img = IMAGE_FIRST_SECTION(NT);

	if (Rva < Img->PointerToRawData)
		return Rva;

	for (i = 0; i < NT->FileHeader.NumberOfSections; i++)
	{
		if (Img[i].SizeOfRawData)
			Limit = Img[i].SizeOfRawData;
		else
			Limit = Img[i].Misc.VirtualSize;

		if (Rva >= Img[i].VirtualAddress &&
		    Rva < (Img[i].VirtualAddress + Limit))
		{
			if (Img[i].PointerToRawData != 0)
			{
				Offset -= Img[i].VirtualAddress;
				Offset += Img[i].PointerToRawData;
			}
				
			return Offset;
		}
	}

	return NULL;
}